Imports System.IO
Imports System.Security.AccessControl
Imports System.Security.Principal
Imports System

Public Class ACL
    ''' <summary>
    ''' Anmelden am einem Windowssystem als anderer Benutzer.
    ''' </summary>
    ''' <param name="lpszUsername"></param>
    ''' <param name="lpszDomain"></param>
    ''' <param name="lpszPassword"></param>
    ''' <param name="dwLogonType"></param>
    ''' <param name="dwLogonProvider"></param>
    ''' <param name="phToken"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Declare Auto Function LogonUser Lib "advapi32.dll" (ByVal lpszUsername As [String], _
              ByVal lpszDomain As [String], ByVal lpszPassword As [String], _
              ByVal dwLogonType As Integer, ByVal dwLogonProvider As Integer, _
              ByRef phToken As IntPtr) As Boolean

    Private Declare Auto Function CloseHandle Lib "kernel32.dll" (ByVal handle As IntPtr) As Boolean

    ''' <summary>
    ''' Meldet sich als aktueller oder Fremder Benutzer am System an. 
    ''' </summary>
    ''' <param name="UserName">Name des Benutzers</param>
    ''' <param name="DomainName">Name der Domain</param>
    ''' <param name="Password">Passwort fr das Benutzerkonto</param>
    ''' <param name="LoginUser">Gibt den Namen des Benutzers zurck </param>
    ''' <param name="UserToken">Gibt die Identitt des Benutzers als IntPtr zurck</param>
    ''' <returns>True=Anmeldung war erfolgreich, False=Anmeldung ist fehlgeschlagen</returns>
    ''' <remarks></remarks>
    Public Shared Function LogIn(ByVal UserName As String, _
                                 ByVal DomainName As String, _
                                 ByVal Password As String, _
                                 ByRef LoginUser As String, _
                                 ByRef UserToken As IntPtr) As Boolean

        Const LOGON32_PROVIDER_DEFAULT As Integer = 0
        Const LOGON32_LOGON_INTERACTIVE As Integer = 2

        Dim ret As Boolean = LogonUser(UserName, DomainName, Password, LOGON32_LOGON_INTERACTIVE, LOGON32_PROVIDER_DEFAULT, UserToken)
        LoginUser = New WindowsIdentity(UserToken).Name

        Return ret

    End Function
    ''' <summary>
    ''' Nimmt die Identitt eines anderen Benutzers an.
    ''' </summary>
    ''' <param name="UserToken">Identitt des Benutzers als IntPtr</param>
    ''' <returns>Stellt die Windows-Benutzer Identitt dar</returns>
    Public Shared Function ImpersonateIdentity(ByVal UserToken As IntPtr) As WindowsImpersonationContext

        Dim NewId As New WindowsIdentity(UserToken)
        Dim impersonationContext As WindowsImpersonationContext = NewId.Impersonate()
        Dim x As String = WindowsIdentity.GetCurrent.Name

        Return impersonationContext
    End Function
    ''' <summary>
    ''' Gibt die angenommene Identitt eines anderen Benutzers zurck.
    ''' </summary>
    ''' <param name="ImpersonateContext">Stellt den Windows-Benutzer vor dem Identittswechsel dar.</param>
    ''' <param name="UserToken">Identitt des Benutzers als IntPtr</param>
    Public Shared Sub LogOut(ByVal ImpersonateContext As WindowsImpersonationContext, ByVal UserToken As IntPtr)
        ImpersonateContext.Undo()

        Dim x As String = WindowsIdentity.GetCurrent.Name
        CloseHandle(UserToken)
    End Sub
    ''' <summary>
    ''' Entfern alle Rechte des Benutzers, und lscht diesen aus der Registerkarte Sicherheit.
    ''' </summary>
    ''' <param name="DomainName">Name der Domain</param>
    ''' <param name="AccountName">Name des Benutzers</param>
    ''' <param name="Path">Name des Ordners</param>
    Public Shared Sub RemoveUserToDirectorySecurity(ByVal DomainName As String, ByVal AccountName As String, ByVal Path As String)
        Dim usr As NTAccount = New NTAccount(String.Format("{0}\{1}", DomainName, AccountName))
        Dim d As DirectoryInfo = New DirectoryInfo(Path)
        Dim dSecurity As DirectorySecurity = d.GetAccessControl()

        dSecurity.RemoveAccessRuleAll(New FileSystemAccessRule(usr, FileSystemRights.FullControl, AccessControlType.Allow))

        d.SetAccessControl(dSecurity)
    End Sub
    ''' <summary>
    ''' Liest die Berechtigung des Benutzers eines Ordners.
    ''' </summary>
    ''' <param name="DomainName">Name der Domain</param>
    ''' <param name="AccountName">Name des Benutzers</param>
    ''' <param name="Path">Name des Ordners</param>
    ''' <returns>Gibt eine List aller Benutzerrechte zurck</returns>
    ''' 
    Public Shared Function ReadUserSecurity(ByVal DomainName As String, ByVal AccountName As String, ByVal Path As String) As List(Of FileSystemRights)
        Dim Rules As New List(Of FileSystemRights)

        Dim UserAcc As NTAccount = New NTAccount(String.Format("{0}\{1}", DomainName, AccountName))
        Dim x As String = UserAcc.Translate(GetType(NTAccount)).ToString
        Dim dInfo As DirectoryInfo = New DirectoryInfo(Path)
        Dim dSecurity As DirectorySecurity = dInfo.GetAccessControl()

        Dim AuthColl As AuthorizationRuleCollection = dSecurity.GetAccessRules(True, True, UserAcc.GetType)
        For Each oRule As FileSystemAccessRule In AuthColl
            Rules.Add(oRule.FileSystemRights)
        Next

        Return Rules
    End Function
    ''' <summary>
    ''' Liest alle Berechtigungen eines Ordners.
    ''' </summary>
    ''' <param name="Path">Name des Ordners z.B.: C:\Test</param>
    ''' <returns>Gibt eine Liste aller Rechte des Ordner Objekts zurck</returns>
    ''' <remarks></remarks>
    Public Shared Function ReadDirectorySecurity(ByVal Path As String) As List(Of String)
        Dim Rules As New List(Of String)
        Dim dirinfo As New DirectoryInfo(Path)

        Dim dsec As DirectorySecurity = dirinfo.GetAccessControl(AccessControlSections.All)

        For Each drule As FileSystemAccessRule In dsec.GetAccessRules(True, True, GetType(NTAccount))
            Rules.Add(drule.FileSystemRights.ToString)
        Next

        Return Rules
    End Function
    ''' <summary>
    ''' Setzt die Berechtigung eines Ordners fr den Benutzer.
    ''' </summary>
    ''' <param name="DomainName">Name der Domain</param>
    ''' <param name="AccountName">Name des Benutzers</param>
    ''' <param name="Path">Name des Ordners z.B.: C:\Test</param>
    ''' <param name="Rights">Rechte die dem Ordner Objekt hinzugefgt werden.</param>
    ''' <remarks></remarks>
    Public Shared Sub SetDirectorySecurity(ByVal DomainName As String, ByVal AccountName As String, ByVal Path As String, ByVal Rights As FileSystemRights)
        Dim dirSecurity As DirectorySecurity = Directory.GetAccessControl(Path)
        Dim User As New NTAccount(String.Format("{0}\{1}", DomainName, AccountName))
        Dim Rule As New FileSystemAccessRule(User, Rights, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, _
                                             PropagationFlags.None, AccessControlType.Allow)

        dirSecurity.AddAccessRule(Rule)

        Dim dirInfo As DirectoryInfo = New DirectoryInfo(Path)
        dirInfo.SetAccessControl(dirSecurity)
    End Sub
    ''' <summary>
    ''' Liest das Token des Angemeldenten Benutzers aus.
    ''' </summary>
    ''' <returns>Gibt das Token des Benutzers als IntPtr zurck</returns>
    ''' <remarks></remarks>
    Public Shared Function UserToken() As IntPtr
        Return WindowsIdentity.GetCurrent.Token
    End Function
    ''' <summary>
    ''' Liest alle Gruppen aus, die der Angemeldete Benutzer besitzt.
    ''' </summary>
    ''' <returns>Gibt eine Liste mit allen Gruppen Namen zurck</returns>
    ''' <remarks></remarks>
    Public Shared Function WindowsGroups() As List(Of WinGroups)
        Dim Groups As New List(Of WinGroups)

        Dim g As IdentityReferenceCollection = WindowsIdentity.GetCurrent.Groups

        For Each ir As IdentityReference In g
            Dim grp As New WinGroups
            grp.SID = ir.Value
            grp.GroupName = ir.Translate(GetType(NTAccount)).Value
            Groups.Add(grp)

        Next

        Return Groups
    End Function
    ''' <summary>
    ''' Lscht Berechtigungen eines Ordners.
    ''' </summary>
    ''' <param name="DomainName">Name der Domain</param>
    ''' <param name="AccountName">Name des Benutzers</param>
    ''' <param name="Path">Name des Ordners z.B.: C:\Test</param>
    ''' <param name="Rights">Welche Rechte entfern werden sollen.</param>
    ''' <param name="ControlType">Zugriff auf das Gesicherte Object zulassen oder verweigern.</param>
    ''' <remarks></remarks>

    Public Shared Sub RemoveDirectorySecurity(ByVal DomainName As String, _
                                       ByVal AccountName As String, _
                                       ByVal Path As String, ByVal Rights As FileSystemRights, _
                                       ByVal ControlType As AccessControlType)

        Dim Account As NTAccount = New NTAccount(String.Format("{0}\{1}", DomainName, AccountName))
        Dim dInfo As New DirectoryInfo(Path)
        Dim dSecurity As DirectorySecurity = dInfo.GetAccessControl()

        dSecurity.RemoveAccessRule(New FileSystemAccessRule(Account, Rights, ControlType))

        dInfo.SetAccessControl(dSecurity)

    End Sub

End Class

#Region "Windows Groups"
''' <summary>
''' Stellt die Eigenschaften SID und Groupname zur verfgung, in Verbindung mit der Methode WindowsGroups
''' </summary>
''' <remarks>Rckgabe der Eigenschaft als Text</remarks>
Public Class WinGroups
    Dim m_SID As String
    Dim m_GrpName As String

    Public Property SID() As String
        Get
            Return m_SID
        End Get
        Set(ByVal value As String)
            m_SID = value
        End Set
    End Property

    Public Property GroupName() As String
        Get
            Return m_GrpName
        End Get
        Set(ByVal value As String)
            m_GrpName = value
        End Set
    End Property
End Class
#End Region

