
#Region "Verweise"
Imports System.DirectoryServices.ActiveDirectory
Imports System.DirectoryServices.DirectorySearcher
Imports System.DirectoryServices
Imports System.Runtime.InteropServices

''' <summary>
''' Stellt verschiedenen Funktionen zur Verfgung um Aktionen der ADS (Active Directory Service) durchzufhren. 
''' </summary>
''' <remarks></remarks>
#End Region
Public Class ADS

#Region "Enumeration fr das Benutzerkonto"

    ''' <summary>
    ''' Steuert das Benutzerkonto.
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum USER_FLAG
        ''' <summary>
        ''' Das Anmeldeskript wird ausgefhrt (Dezimalwert:1)
        ''' </summary>
        ''' <remarks></remarks>
        ADS_UF_SCRIPT = &H1
        ''' <summary>
        ''' Das Benutzerkonto wird deaktiviert(Dezimalwert:2)
        ''' </summary>
        ''' <remarks></remarks>
        ADS_UF_ACCOUNTDISABLE = &H2

        'ADS_UF_HOMEDIR_REQUIRED = &H8 'Basisverzeichnis erforderlich
        'ADS_UF_LOCKOUT = &H10 'Es ist kein Kennwort erforderlich
        'ADS_UF_PASSWD_NOTREQD = &H20 'Es ist kein Kennwort erforderlich

        ''' <summary>
        ''' Der Benutzer kann das Kennwort nicht ndern (Dezimalwert:64)
        ''' </summary>
        ''' <remarks></remarks>
        ADS_UF_PASSWD_CANT_CHANGE = &H40

        'ADS_UF_ENCRYPTED_TEXT_PASSWORD_ALLOWED = &H80 'Der Benutzer kann ein verschlsseltes Kennwort senden
        'ADS_UF_TEMP_DUPLICATE_ACCOUNT = &H100 'Konto fr Benutzer, deren primres Konto sich in einer anderen Domne befindet

        ''' <summary>
        ''' Standardkontotyp fr einen typischen Benutzer (Dezimalwert:512)
        ''' </summary>
        ''' <remarks></remarks>
        ADS_UF_NORMAL_ACCOUNT = &H200

        'ADS_UF_INTERDOMAIN_TRUST_ACCOUNT = &H800 'Konto fr ein Vertrauenskonto einer Domne, die anderen Domnen vertraut
        'ADS_UF_WORKSTATION_TRUST_ACCOUNT = &H1000 'Computerkonto fr einen Computer mit Microsoft Windows NT 4.0 Workstation
        'ADS_UF_SERVER_TRUST_ACCOUNT = &H2000 'Computerkonto fr einen Domnencontroller, der Mitglied dieser Domne ist

        ''' <summary>
        ''' Kennwort, das fr dieses Konto nie ablaufen sollte (Dezimalwert:65536)
        ''' </summary>
        ''' <remarks></remarks>
        ADS_UF_DONT_EXPIRE_PASSWD = &H10000

        'ADS_UF_MNS_LOGON_ACCOUNT = &H20000 'MNS- Anmeldekonto
        'ADS_UF_SMARTCARD_REQUIRED = &H40000 'Wenn dieses Kennzeichen gesetzt ist, wird der Benutzer gezwungen, sich ber eine Smartcard anzumelden
        'ADS_UF_TRUSTED_FOR_DELEGATION = &H80000
        'ADS_UF_NOT_DELEGATED = &H100000
        'ADS_UF_USE_DES_KEY_ONLY = &H200000
        'ADS_UF_DONT_REQUIRE_PREAUTH = &H400000

        ''' <summary>
        ''' Das Kennwort des Benutzers ist abgelaufen (Dezimalwert:8388608)
        ''' </summary>
        ''' <remarks></remarks>
        ADS_UF_PASSWORD_EXPIRED = &H800000

        'ADS_UF_TRUSTED_TO_AUTHENTICATE_FOR_DELEGATION = &H1000000

        'Weitere Information bei: http://support.microsoft.com/kb/305144/de
    End Enum
#End Region

#Region "Funktion TerminalServer"
    Public Enum WTS_CONFIG_CLASS
        'WTSUserConfigInitialProgram
        'WTSUserConfigWorkingDirectory
        'WTSUserConfigfInheritInitialProgram
        'WTSUserConfigfAllowLogonTerminalServer
        'WTSUserConfigTimeoutSettingsConnections
        'WTSUserConfigTimeoutSettingsDisconnections
        'WTSUserConfigTimeoutSettingsIdle
        'WTSUserConfigfDeviceClientDrives
        'WTSUserConfigfDeviceClientPrinters
        'WTSUserConfigfDeviceClientDefaultPrinter
        'WTSUserConfigBrokenTimeoutSettings
        'WTSUserConfigReconnectSettings
        'WTSUserConfigModemCallbackSettings
        'WTSUserConfigModemCallbackPhoneNumber
        'WTSUserConfigShadowingSettings
        WTSUserConfigTerminalServerProfilePath = &HF
        WTSUserConfigTerminalServerHomeDir = &H10
        WTSUserConfigTerminalServerHomeDirDrive = &H11
        'WTSUserConfigfTerminalServerRemoteHomeDir = &H12
    End Enum
    <DllImport("wtsapi32.dll", CharSet:=CharSet.Auto)> _
  Private Shared Function WTSSetUserConfig( _
      <MarshalAs(UnmanagedType.LPTStr)> ByVal pServerName As String, _
      <MarshalAs(UnmanagedType.LPTStr)> ByVal pUserName As String, _
      ByVal WTSConfigClass As WTS_CONFIG_CLASS, _
      <MarshalAs(UnmanagedType.LPTStr)> ByVal pBuffer As String, _
      ByVal DataLength As Integer) As Integer
    End Function
#End Region

#Region "Hinzufgen Aktionen"

    ''' <summary>Fgt das Benutzerkonto zu einer bestimmten Gruppe hinzu.</summary>
    ''' <param name="Domain">Name des Benutzers aus der Domain</param>
    ''' <param name="DomainUser">Benutzerkonto als System.DirectoryServices.DirectoryEntry</param>
    ''' <param name="GroupName">Gruppen Name aus der Domain</param>
    '''<returns> True oder False</returns>
    Public Shared Function AddUserToGroup(ByVal Domain As DirectoryEntry, ByVal DomainUser As DirectoryEntry, ByVal GroupName As String) As Boolean

        Dim DomainSearch As DirectorySearcher = New DirectorySearcher()
        DomainSearch.SearchRoot = Domain
        DomainSearch.Filter = "(&(objectClass=group) (CN=" & GroupName & "))"
        Dim Result As SearchResult = DomainSearch.FindOne()

        Dim Results As SearchResultCollection = DomainSearch.FindAll()

        Dim isGroupMember As Boolean = False

        If Results.Count > 0 Then
            Dim Group As DirectoryEntry = Result.GetDirectoryEntry()
            Dim Groups As DirectoryEntry = New DirectoryEntry(Results(0).Path)
            Dim Members As Object = Groups.Invoke("Members", Nothing)

            For Each Member As Object In CType(Members, IEnumerable)

                Dim x As DirectoryEntry = New DirectoryEntry(Member)
                Dim Name As String = x.Name.ToUpper

                If Name <> DomainUser.Name.ToUpper Then
                    isGroupMember = False
                Else
                    isGroupMember = True
                    Exit For
                End If
            Next Member

            Try
                If (Not isGroupMember) Then

                    Group.Invoke("Add", New Object() {DomainUser.Path.ToString()})
                    Group.CommitChanges()
                End If
            Catch ex As Exception
                Return False
            End Try
        End If

        Return True
    End Function

    ''' <summary>
    ''' Fgt ein Benutzerkonto der Domain hinzu.
    ''' </summary>
    ''' <param name="UserName">Name des Benutzers aus der Domain</param>
    ''' <returns>Gibt True=OK oder False=Fehler zurck</returns>
    ''' <remarks></remarks>
    Public Shared Function AddUser(ByVal Domain As DirectoryEntry, ByVal UserName As String, ByVal Password As String, ByVal PropertyList As List(Of UserProperty)) As Boolean
        Dim Users As DirectoryEntries = GetDomain.Children()
        Dim NewUser As DirectoryEntry = Users.Add("CN=" & UserName, "user")

        Try

            For i As Integer = 0 To PropertyList.Count - 1
                SetProperty(NewUser, PropertyList.Item(i).Name, UserName.ToString)

                NewUser.CommitChanges()
            Next

            EnabledUserAccount(NewUser, True)

            SetPassword(NewUser, Password)
            Return True
        Catch ex As Exception
            Return False
        Finally
            NewUser.Close()
            NewUser.Dispose()
        End Try

    End Function

    ''' <summary>
    ''' Erstellt eine neue Gruppe in der Domain.
    ''' </summary>
    ''' <param name="Domain">Domain Objekt als DirectoryEntry</param>
    ''' <param name="GroupName">Name der Gruppe</param>
    ''' <returns>True oder False</returns>
    ''' <remarks></remarks>
    Public Shared Function AddGroup(ByVal Domain As DirectoryEntry, ByVal GroupName As String, ByVal Description As String) As Boolean

        Dim Group As DirectoryEntries = Domain.Children()
        With Group
            Dim NewGroup As DirectoryEntry = .Add("CN=" & GroupName, "group")

            With NewGroup
                SetProperty(NewGroup, "SAMAccountName", GroupName)
                SetProperty(NewGroup, "Description", Description)
                .CommitChanges()
            End With
            NewGroup.Close()
            NewGroup.Dispose()
        End With
    End Function
#End Region

#Region "Get Aktionen"
    ''' <summary>
    ''' Stellt eine Verbindung mit LDAP:// zur Domaine her
    ''' </summary>
    ''' <param name="UserName">Benutzername des Administarators der vollzugriff in der Domain hat</param>
    ''' <param name="Password">Passwort des Administrators</param>
    ''' <param name="Organisation">Organisationseinheit in der Domain z.B.:CN=Users</param>
    ''' <param name="DomainController">Name des Computers der die Domain reprsentiert</param>
    ''' <param name="Suffix">WebLnderkennzeichen z.B.: com</param>
    ''' <returns>Domain Objekt als DirectoryEntry</returns>
    ''' <remarks></remarks>

    Public Shared Function GetDomain(Optional ByVal UserName As String = "Benutzername", _
                                     Optional ByVal Password As String = "Passwort", _
                                     Optional ByVal Organisation As String = "CN=Users", _
                                     Optional ByVal DomainController As String = "DC=meine-Sonne", _
                                     Optional ByVal Suffix As String = "DC=com") As DirectoryEntry

        Dim Domain As New DirectoryEntry()
        Dim MyPath As String = String.Format("{0}://{1},{2},{3}", "LDAP", Organisation, DomainController, Suffix)
        Domain.Path = MyPath
        Domain.Username = UserName
        Domain.Password = Password
        Domain.AuthenticationType = AuthenticationTypes.Secure
        Return Domain

    End Function
    ''' <summary>
    '''  Stellt die Verbindung mit WinNT:// zu einem Benutzerkonto in der Domaine her.
    ''' </summary>
    ''' <param name="ServerName">Name des Domain Servers</param>
    ''' <param name="UserName">Name des Benutzers aus der Domain</param>
    ''' <param name="Organisation">Organisationseinheit in der Domain z.B.:Computer</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetUser(ByVal ServerName As String, ByVal UserName As String, Optional ByVal Organisation As String = "computer") As DirectoryEntry
        Dim MyPath As String = String.Format("WinNT://{0},{1}", ServerName, Organisation)
        Dim Domain As New DirectoryEntry(MyPath)
        Return Domain.Children.Find(UserName, "user")
    End Function

    ''' <summary>
    ''' Stellt die Verbindung zu einem Benutzerkonto in der Domaine her.
    ''' </summary>
    ''' <param name="Domain">Domain Object als System.DirectoryService.DirectoryEntry</param>
    ''' <param name="UserName">Name des Benutzers aus der Domain</param>
    ''' <returns>Gibt das Benutzerobjekt als System.DirectoryServices.DirectoryEntry zurck.  </returns>
    ''' <remarks></remarks>

    Public Shared Function GetUser(ByVal Domain As DirectoryEntry, ByVal UserName As String) As DirectoryEntry
        Dim User As DirectoryEntry = Domain.Children.Find("CN=" & UserName)
        Return User
    End Function
    ''' <summary>
    ''' Liest die entsprechende Kategorie der Domain aus.
    ''' </summary>
    ''' <param name="Category">Die abzurufende Kategorie z.B.: Computer,oder User,oder Group</param>
    ''' <param name="CN">Bestimmten Eintrag der Kategorie auslesen. z.B.: "TestUser"</param>
    ''' <param name="Filter">Was soll angezeigt werden. z.B.: Bei Computer kann es mehrere OU= geben,
    ''' Es werden alle Computer bei denen die OU=Rechner ist zurckgegeben.
    ''' </param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetCategoryBy(ByVal Category As String, Optional ByVal CN As String = "*", Optional ByVal Filter As String = "") As List(Of String)
        Dim DomainSearche As DirectorySearcher = New DirectorySearcher()
        DomainSearche.Filter = "(&(objectCategory=" & Category & ")(CN=" & CN & "))"
        Dim CategoryList As New List(Of String)
        Dim AllCategorys As SearchResultCollection = DomainSearche.FindAll()

        For Each x As SearchResult In AllCategorys
            If x.Path.IndexOf(Filter) > -1 Then
                CategoryList.Add(x.Path.Substring(0, x.Path.IndexOf(",")).Replace("LDAP://CN=", ""))
            End If
        Next
        CategoryList.Sort()
        Return CategoryList
    End Function
    ''' <summary>
    ''' Liest alle verfgbaren Eigenschaften des Domain Benutzers aus.
    ''' </summary>
    ''' <param name="User">Fordert das Benutzerobjekt als System.DirectoryServices.DirectoryEntry</param>
    ''' <returns>Gibt eine Liste der Eigenschaften als 'List(Of UserProperties)' zurck</returns>
    ''' <remarks></remarks>
    Public Shared Function GetUserProperties(ByVal User As DirectoryEntry) As List(Of UserProperty)

        Dim myProperties As New List(Of UserProperty)

        For Each s As PropertyValueCollection In User.Properties
            Dim PropValues As String = User.Properties.Item(s.PropertyName.ToString).Value.ToString

            If PropValues.ToUpper.IndexOf("SYSTEM") = -1 Then
                Dim Propertie As New UserProperty
                Propertie.Name = s.PropertyName.ToUpper.ToString
                Propertie.Value = s.Value.ToString
                myProperties.Add(Propertie)
            End If
        Next

        Return myProperties

    End Function

    ''' <summary>
    ''' Liest alle Domain Controller aus der Domaine aus.
    ''' </summary>
    ''' <returns>Gibt eine Liste der Domain Controller als 'List(Of String)' zurck</returns>
    ''' <remarks></remarks>
    Public Shared Function GetAllDomainControllers() As List(Of String)
        Dim DomainList As New List(Of String)
        Dim listeDC As DomainControllerCollection = Nothing
        Dim monContext As New DirectoryContext(DirectoryContextType.Domain, Domain.GetCurrentDomain.ToString)
        Dim monDomaine As Domain = Domain.GetDomain(monContext)
        listeDC = monDomaine.FindAllDomainControllers()

        For Each DomController As DomainController In listeDC
            DomainList.Add(DomController.Name.ToUpper.Substring(0, DomController.Name.IndexOf(".")))
        Next
        DomainList.Sort()
        Return DomainList

    End Function
    ''' <summary>
    ''' Liest alle Gruppen der Domain aus.
    ''' </summary>
    ''' <returns>Gibt eine Liste der Gruppen aus der Domain als 'List(Of String)' zurck</returns>
    ''' <remarks></remarks>
    Public Shared Function GetAllGroups() As List(Of String)
        Dim GroupList As New List(Of String)
        Dim DomainSearch As DirectorySearcher = New DirectorySearcher("objectCategory=group")
        Dim Results As SearchResultCollection = DomainSearch.FindAll()

        For Each Group As SearchResult In Results
            Dim grpName As String = Group.Path.Substring(0, Group.Path.IndexOf(",")).Replace("LDAP://CN=", "")
            GroupList.Add(grpName)
        Next
        GroupList.Sort()
        Return GroupList
    End Function

    ''' <summary>
    ''' Liest alle Gruppen aus, die dem Benutzerkonto zugeordnet sind.
    ''' </summary>
    ''' <param name="UserName">Name des Benutzers aus der Domain</param>
    ''' <returns>Gibt alle Gruppen aus der Domaine die dem Benutzerkonto zugeordnet wurde in einer List(Of String) zurck.</returns>
    Public Shared Function GetUserInGroups(ByVal Domain As DirectoryEntry, ByVal UserName As String) As List(Of String)
        Dim GroupList As New List(Of String)
        Dim sFilter As String = String.Format("(sAMAccountName={0})", UserName)
        Dim DomainSearcher As DirectorySearcher = New DirectorySearcher(Domain, sFilter)
        Dim oResult As SearchResult = DomainSearcher.FindOne

        If Not (oResult Is Nothing) Then
            Dim obUser As DirectoryEntry = New DirectoryEntry(oResult.Path)
            Dim obGroups As Object = obUser.Invoke("Groups")
            For Each Group As Object In DirectCast(obGroups, IEnumerable)
                Dim obGpEntry As DirectoryEntry = New DirectoryEntry(Group)
                GroupList.Add(obGpEntry.Name.ToUpper.Replace("CN=", ""))
            Next
        End If
        GroupList.Sort()
        Return GroupList
    End Function
#End Region

#Region "Set Aktionen"
    ''' <summary>
    ''' Aktiviert oder Deaktiviert das Benutzerkonto.
    ''' </summary>
    ''' <param name="User">Fordert das Benutzerobjekt als System.DirectoryServices.DirectoryEntry</param>
    ''' <param name="Flags">Fordert einen Eintrag aus dem Enummeration USER_FLAG  </param>
    ''' <remarks></remarks>
    Public Shared Sub SetUserAccount(ByVal User As DirectoryEntry, ByVal Flags As Integer)
        Dim val As Integer
        val = Fix(User.Properties("userFlags").Value)
        User.Properties("userFlags").Value = val Or Flags
        User.CommitChanges()
    End Sub
    ''' <summary>
    ''' Weist dem Benutzerkonto Eigenschaften aus einer Vordefinierten List(Of UserPropertys) zu.
    ''' </summary>
    ''' <param name="Domain">Name der Domain</param>
    ''' <param name="User">Benutzerkonto als DirectoryEntry</param>
    ''' <remarks></remarks>
    Public Shared Sub SetSpecialUserPropertys(ByVal Domain As DirectoryEntry, ByVal User As DirectoryEntry)
        Dim UserProperies As List(Of UserProperty) = ListSpecialProperties()

        For i As Integer = 0 To UserProperies.Count - 1
            SetProperty(User, UserProperies.Item(i).Name, UserProperies.Item(i).Value)
            User.CommitChanges()
        Next
    End Sub

    ''' <summary>
    ''' ndert das Passwort fr ein Benutzerkonto.
    ''' </summary>
    ''' <param name="User">Benutzers als DirectoryEntry</param>
    ''' <param name="PWS">Das neue Passowrt fr das Benutzerkonto</param>
    ''' <remarks></remarks>
    Public Shared Sub SetPassword(ByVal User As DirectoryEntry, ByVal PWS As String)
        'Dim User As DirectoryEntry = New DirectoryEntry()
        'User.Path = Path
        'User.AuthenticationType = AuthenticationTypes.Secure
        'User.Password = PWS
        User.Invoke("SetPassword", PWS)
        User.CommitChanges()
    End Sub
    ''' <summary>
    ''' Setzt den Terminal Server Profil Pfad fr ein Benutzerkonto
    ''' </summary>
    ''' <param name="ServerName">Name des Domain Servers z.B.: \\Sonne</param>
    ''' <param name="UserName">Name des Benutzers in der Domain</param>
    ''' <param name="TerminalServerProfilePath">Pfad zum Profile Verzeichnis z.B.: \\vsprofile\profile\%username%</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function SetTeminalProfilePath(ByVal ServerName As String, ByVal UserName As String, ByVal TSConfig As WTS_CONFIG_CLASS, ByVal TerminalServerProfilePath As String) As Integer
        'Dim Pr As New functions()
        'Return Pr.SetTSProfilePath(ServerName, UserName, TerminalServerProfilePath)

        Return WTSSetUserConfig(ServerName, UserName, TSConfig, TerminalServerProfilePath, TerminalServerProfilePath.Length)
    End Function

#End Region

#Region "ndern Aktionen"
    ''' <summary>
    ''' ndert die Eigenschaften  des Benutzerskontos.
    ''' </summary>
    ''' <param name="Domain">Domain Objekt als System.DirectoryService.DirectoryEntry</param>
    ''' <param name="UserName">Name des Benutzers aus der Domain</param>
    ''' <returns>True oder False</returns>
    ''' <remarks></remarks>
    Public Shared Function ChangeUserProperties(ByVal Domain As DirectoryEntry, ByVal UserName As String, ByVal PropertyList As List(Of UserProperty)) As Boolean
        'Alle Propeties prfen, als liste bergeben und fllen mit schleife
        Dim ReadProperty As PropertyValueCollection
        Dim User As DirectoryEntries = Domain.Children
        Dim myUser As DirectoryEntry = User.Find("CN=" & UserName)

        For i As Integer = 0 To PropertyList.Count - 1
            ReadProperty = myUser.Properties(PropertyList.Item(i).Name)

            With ReadProperty
                .Clear()
                .Add(PropertyList.Item(i).Value)
            End With
            myUser.CommitChanges()
        Next

        myUser.Close()
        myUser.Dispose()
        Domain.Close()
        Domain.Dispose()

    End Function
    ''' <summary>
    ''' Kopiert einen vorhandenens Benutzerkonto und weist die Eigenschaften einem Neuen Benuzterkonto zu.
    ''' </summary>
    ''' <param name="Domain">Domain Objekt als System.DirectoryService.DirectoryEntry</param>
    ''' <param name="UserReferenz">Bestenhenders Benutzerkonto der Active Directory</param>
    ''' <param name="NewUserName">Name des Neuen Benutzerkontos</param>
    ''' <param name="Password">Passwort fr das Benutzerkonto</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function CloneUserAccount(ByVal Domain As DirectoryEntry, ByVal UserReferenz As DirectoryEntry, ByVal NewUserName As String, ByVal Password As String) As Boolean
        Dim UserProperies As List(Of UserProperty) = ListSpecialProperties()
        Dim UserGroups As List(Of String) = GetUserInGroups(Domain, UserReferenz.Name)

        'UserProperies.Item(0).Value = NewUserName

        If IsUserExists(Domain, NewUserName) = False Then
            If UserProperies.Count > 0 Then
                If AddUser(Domain, NewUserName, Password, UserProperies) = True Then
                    For i As Integer = 0 To UserGroups.Count - 1
                        AddUserToGroup(Domain, UserReferenz, UserGroups.Item(i).ToString)
                    Next
                End If
            End If
        Else

        End If
    End Function

#End Region

#Region "Lsch Aktionen"
    ''' <summary>
    ''' Lscht ein Benutzerkonto aus der Domain.
    ''' </summary>
    ''' <param name="UserName">Name des Benutzers aus der Domain</param>
    ''' <returns>True oder False</returns>
    ''' <remarks></remarks>
    Public Shared Function RemoveUser(ByVal Domain As DirectoryEntry, ByVal UserName As String) As Boolean
        Dim deDomain As DirectoryEntries = Domain.Children
        Dim deUser As DirectoryEntry = deDomain.Find("CN=" & UserName)
        Try
            deDomain.Remove(deUser)

        Catch ex As Exception
            Return False
        Finally
        End Try

        Return True
    End Function


    ''' <summary>Lscht das Benutzerkonto in einer bestimmten Gruppe.</summary>
    ''' <param name="Domain">Name des Benutzers aus der Domain</param>
    ''' <param name="DomainUser">Benutzerkonto als System.DirectoryServices.DirectoryEntry</param>
    ''' <param name="GroupName">Gruppen Name aus der Domain</param>
    ''' <returns>True oder False</returns>
    Public Shared Function RemoveUserToGroup(ByVal Domain As DirectoryEntry, ByVal DomainUser As DirectoryEntry, ByVal GroupName As String) As Boolean

        Dim DomainSearch As DirectorySearcher = New DirectorySearcher()
        DomainSearch.SearchRoot = Domain
        DomainSearch.Filter = "(&(objectClass=group) (CN=" & GroupName & "))"
        Dim Result As SearchResult = DomainSearch.FindOne()

        Dim Results As SearchResultCollection = DomainSearch.FindAll()

        Dim isGroupMember As Boolean = False

        If Results.Count > 0 Then
            Dim Group As DirectoryEntry = Result.GetDirectoryEntry()
            Dim Groups As DirectoryEntry = New DirectoryEntry(Results(0).Path)
            Dim Members As Object = Groups.Invoke("Members", Nothing)

            For Each Member As Object In CType(Members, IEnumerable)

                Dim x As DirectoryEntry = New DirectoryEntry(Member)
                Dim Name As String = x.Name.ToUpper
                If Name = DomainUser.Name.ToUpper Then
                    isGroupMember = True
                    Exit For
                End If
            Next Member

            Try
                If (isGroupMember) Then
                    Group.Invoke("Remove", New Object() {DomainUser.Path.ToString()})
                    Group.CommitChanges()
                End If

            Catch ex As Exception
                Return False
            End Try

        End If

        Return True
    End Function

#End Region

#Region "Aktion Prfungen (Is)"
    ''' <summary>Ist das Benutzerkonto in einer bestimmten Gruppe.</summary>
    ''' <param name="Domain">Name des Benutzers aus der Domain</param>
    ''' <param name="DomainUser">Benutzerkonto als System.DirectoryServices.DirectoryEntry</param>
    ''' <param name="GroupName">Gruppen Name aus der Domain</param>
    ''' <returns>True oder False</returns>
    Public Shared Function IsUserInGroup(ByVal Domain As DirectoryEntry, ByVal DomainUser As DirectoryEntry, ByVal GroupName As String) As Boolean

        Dim DomainSearch As DirectorySearcher = New DirectorySearcher()
        DomainSearch.SearchRoot = Domain
        DomainSearch.Filter = "(&(objectClass=group) (CN=" & GroupName & "))"

        Dim Results As SearchResultCollection = DomainSearch.FindAll()

        Dim isGroupMember As Boolean = False

        If Results.Count > 0 Then
            Dim Groups As DirectoryEntry = New DirectoryEntry(Results(0).Path)
            Dim Members As Object = Groups.Invoke("Members", Nothing)

            For Each Member As Object In CType(Members, IEnumerable)
                Dim x As DirectoryEntry = New DirectoryEntry(Member)
                If x.Name.ToUpper = DomainUser.Name.ToUpper Then
                    isGroupMember = True
                    Exit For
                End If
            Next Member

        End If

        Return isGroupMember
    End Function

    ''' <summary>
    ''' Existiert das Benutzerkonto in der Domain
    ''' </summary>
    ''' <param name="UserName">Name des Benutzers aus der Domain</param>
    ''' <returns>Gibt True=Benutzerkonto vorhanden oder False=Benutzerkonto nicht vorhanden zurck</returns>
    Public Shared Function IsUserExists(ByVal Domain As DirectoryEntry, ByVal UserName As String) As Boolean

        Dim DomainSearch As DirectorySearcher = New DirectorySearcher()
        DomainSearch.SearchRoot = Domain
        DomainSearch.Filter = "(&(objectClass=user) (CN=" & UserName & "))"
        Dim Results As SearchResultCollection = DomainSearch.FindAll()

        If Results.Count = 0 Then
            Return False
        Else
            Return True
        End If

    End Function

    ''' <summary>
    ''' Existiert das Benutzerkonto in der Domain.
    ''' </summary>
    ''' <param name="DomainName">Der Name der Domain z.B.: Microsoft</param>
    ''' <param name="ComputerName">Name des Servers in der Domain</param>
    ''' <param name="UserName">Name des Benutzerkontos </param>
    ''' <returns>True oder False</returns>
    Public Shared Function IsUserExists(ByVal DomainName As String, ByVal ComputerName As String, ByVal UserName As String) As Boolean
        Dim myPath As String = String.Format("{0}://{1}/{2}/{3}", "WinNT", DomainName.ToUpper, ComputerName.ToUpper, UserName)
        Dim Domain As New DirectoryEntry(myPath)
        Return DirectoryEntry.Exists(Domain.Path)
    End Function
    ''' <summary>
    ''' Existiert die Gruppe in der Domain.
    ''' </summary>
    ''' <param name="DomainName">Der Name der Domain z.B.: Microsoft</param>
    ''' <param name="ComputerName">Name des Servers in der Domain</param>
    ''' <param name="GroupName">Name der Gruppe </param>
    ''' <returns>True oder False</returns>
    Public Shared Function IsGroupExists(ByVal DomainName As String, ByVal ComputerName As String, ByVal GroupName As String) As Boolean
        Dim myPath As String = String.Format("{0}://{1}/{2}/{3}", "WinNT", DomainName.ToUpper, ComputerName.ToUpper, GroupName)
        Dim Domain As New DirectoryEntry(myPath)
        Return DirectoryEntry.Exists(Domain.Path)
    End Function
#End Region

#Region "Spezial Funktionen"
    ''' <summary>
    ''' Aktiviert ein Benutzerkonto.
    ''' </summary>
    ''' <param name="User">Benutzerkonto als DirectoryEntry</param>
    ''' <param name="Enabled">True fr aktivieren, False fr deaktivieren</param>
    ''' <remarks></remarks>
    Public Shared Sub EnabledUserAccount(ByVal User As DirectoryEntry, ByVal Enabled As Boolean)
        User.NativeObject.AccountDisabled = Enabled
        User.CommitChanges()
        User.Close()
        User.Dispose()
    End Sub


    Private Shared Sub SetProperty(ByVal de As DirectoryEntry, ByVal PropertyName As String, ByVal PropertyValue As String)
        If Not PropertyValue Is Nothing Then
            If de.Properties.Contains(PropertyName) = True Then
                de.Properties(PropertyName)(0) = PropertyValue
            Else
                de.Properties(PropertyName).Add(PropertyValue)
            End If
        End If
    End Sub

    ''' <summary>
    ''' Gibt vordefinierte Benutzereigenschaften in einer List(Of UserPropertys) zurck.
    ''' </summary>
    ''' <returns>Liste von benutzerdefinierten Eigenschaften</returns>
    ''' <remarks></remarks>
    Public Shared Function ListSpecialProperties() As List(Of UserProperty)
        Dim PropertyList As New List(Of UserProperty)
        Dim x As New UserProperty("C", "DE")
        PropertyList.Add(x)

        x = New UserProperty("L", "Musterhausen")
        PropertyList.Add(x)

        x = New UserProperty("ST", "Mustermann")
        PropertyList.Add(x)

        x = New UserProperty("TITLE", "Herr")
        PropertyList.Add(x)

        x = New UserProperty("postalcode", "88131")
        PropertyList.Add(x)

        x = New UserProperty("PHYSICALDELIVERYOFFICENAME", "EDV")
        PropertyList.Add(x)

        x = New UserProperty("TELEPHONENUMBER", "+49 000 77789")
        PropertyList.Add(x)

        x = New UserProperty("FACSIMILETELEPHONENUMBER", "+49 000 14789")
        PropertyList.Add(x)

        x = New UserProperty("GIVENNAME", "Testcx5")
        PropertyList.Add(x)

        x = New UserProperty("INITIALS", "TD")
        PropertyList.Add(x)

        x = New UserProperty("DISPLAYNAME", "Test CX5")
        PropertyList.Add(x)

        x = New UserProperty("DEPARTMENT", "EDV")
        PropertyList.Add(x)

        x = New UserProperty("COMPANY", "Company")
        PropertyList.Add(x)

        x = New UserProperty("STREETADDRESS", "Test Str. 119")
        PropertyList.Add(x)

        x = New UserProperty("WWWHOMEPAGE", "www.lala.com")
        PropertyList.Add(x)

        x = New UserProperty("sn", "CX5")
        PropertyList.Add(x)

        x = New UserProperty("CO", "GERMANY")
        PropertyList.Add(x)

        x = New UserProperty("EMPLOYEEID", 8830)
        PropertyList.Add(x)

        x = New UserProperty("scriptPath", "login.cmd")
        PropertyList.Add(x)

        x = New UserProperty("DIVISION", "EDV")
        PropertyList.Add(x)

        x = New UserProperty("MAIL", "test.cx5@lala.com")
        PropertyList.Add(x)

        x = New UserProperty("HOMEPHONE", "+49 000 12345")
        PropertyList.Add(x)

        x = New UserProperty("MOBILE", "+49 000 1234567")
        PropertyList.Add(x)


        Return PropertyList
    End Function
#End Region

End Class

#Region "Benutzereigenschaften der ADS"
''' <summary>
''' Stellt Eigenschaften zur Verfgung
''' </summary>
''' <remarks></remarks>
Public Class UserProperty
    Dim m_PropertyName As String
    Dim m_PropertyValue As String
    ''' <summary>
    ''' Erstellt eine neue Instanz der Klasse UserProperty
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
    End Sub
    ''' <summary>
    ''' Estellt eine neue Instanz von UserProperty mit bergabeparameter
    ''' </summary>
    ''' <param name="PropertyName">Name des Eigenschaftfeldes</param>
    ''' <param name="PropertyValue">Inhalt des des Eigenschaftfeldes</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal PropertyName As String, ByVal PropertyValue As String)
        Me.Name = PropertyName
        Me.Value = PropertyValue
    End Sub
    ''' <summary>
    ''' Name der Eigenschaft des Benutzerkontos
    ''' </summary>
    ''' <value></value>
    ''' <returns>Gibt den Name des Eigenschaftfeldes zurck.</returns>
    ''' <remarks></remarks>
    Public Property Name() As String
        Get
            Return m_PropertyName
        End Get
        Set(ByVal value As String)
            m_PropertyName = value
        End Set
    End Property
    ''' <summary>
    ''' Inhalt der Eigenschaft des Benutzerkontos
    ''' </summary>
    ''' <value></value>
    ''' <returns>Gibt den Inhalt des Eigenschaftfeldes zurck.</returns>
    ''' <remarks></remarks>
    Public Property Value() As String
        Get
            Return m_PropertyValue
        End Get
        Set(ByVal valu As String)
            m_PropertyValue = valu
        End Set
    End Property

End Class
#End Region